autowatch = 1;


// ver 0.9

var name = "export";
var objext = ".obj";
var filename = name + objext;
var groupmode = 0;
var primitive = "tri_grid";
var pc = 0;
var matsize = 0;
var vertoffset = 0;
var active = 0;
var folder = "";
var mtlext = ".mtl";

var mtlname = name;

//material parameters
var hasmaterial = 0;
var groupnum = 0;

var ambcolor = new Array();
ambcolor[0] = 0.2;
ambcolor[1] = 0.2;
ambcolor[2] = 0.2;

var diffcolor = new Array();
diffcolor[0] = 0.8;
diffcolor[1] = 0.8;
diffcolor[2] = 0.8;

var speccolor = new Array();
speccolor [0] = 1.0;
speccolor [1] = 1.0;
speccolor [2] = 1.0;

var alpha = 1.0;

var shininess = 0.0;
var illum = 1;
var texmap = "export.tif";

//////////////////////////

function checkext(str)
{
    return str.substring(str.length-4, str.length) == objext;
}

function face(i)
{
    i += vertoffset;
    var s = "";
    if (pc == 3)
        s = i;
    else if (pc == 5)
        s = doublet(i);
    else if (pc >= 8)
        s = triplet(i);
    else
    {
        post("Error: Please use a planecount of 3, 5, 8 or more than 8");
        return 0;
    }
    return s;
}

function triplet(i)
{
    var s = i + "/" + i + "/" + i;
    return s
}

function doublet(i)
{
    var s = i + "/" + i;
    return s;
}

function getoffset(file)
{
    while(file.position != file.eof)
    {
        line = file.readline();
        if (line.match("#vertex count: "))
        {
            s = line.substring(line.indexOf(':')+1, line.length);
            offset =  parseInt(s);
            file.position -= line.length+1;
            return offset;
        }
    }
}

////////////////////////////////////////

function settexture(tex)
{
    texmap = tex;
}

function setmaterial(ambr, ambg, ambb, diffr, diffg, diffb, specr, specg, specb, a, ill, shin)
{
    ambcolor[0] = ambr;
    ambcolor[1] = ambg;
    ambcolor[2] = ambb;
    
    diffcolor[0] = diffr;
    diffcolor[1] = diffg;
    diffcolor[2] = diffb;
    
    speccolor[0] = specr;
    speccolor[1] = specg;
    speccolor[2] = specb;
    
    alpha = a;
    shininess = shin;
    illum = ill;
}

function writematerial(x)
{
    hasmaterial = x;
}

function setgroupmode(mode)
{
    groupmode = mode;
}

function setgroupnumber(num)
{
    groupnum = num;
}

function setprimitive(primitivename)
{
    primitive = primitivename;
}

function setfilename(name)
{
    if (checkext(name))
    {
        filename = name;
    folder = name.substring(0, name.lastIndexOf('/')+1);
        post("Target file: " + filename + "\n");
        post("Target folder: " + folder + "\n");
        mtlname = name.substring(name.lastIndexOf("/")+1, name.length-4);
    }
}

function writeMaterialFile()
{
    var mtlfile = new File(folder + mtlname + mtlext, "write");
    mtlfile.open();
    
    if (!mtlfile.isopen)
    {
        post("Error: Failed to open file " + mtlname);
        return;
    }
    
    mtlfile.writestring("newmtl " + mtlname + "\n");
    
    mtlfile.writestring("Ka " + ambcolor[0] + " " + ambcolor[1] + " " + ambcolor[2] + "\n");
    mtlfile.writestring("Kd " + diffcolor[0] + " " + diffcolor[1] + " " + diffcolor[2] + "\n");
    mtlfile.writestring("Ks " + speccolor[0] + " " + speccolor[1] + " " + speccolor[2] + "\n");
    mtlfile.writestring("illum " + illum + "\n");
    mtlfile.writestring("d " + alpha + "\n");
    mtlfile.writestring("Ns " + shininess + "\n");
    mtlfile.writestring("map_Kd " + texmap + "\n");
    
    mtlfile.eof = mtlfile.position;
    mtlfile.close();
}
                

//// jit_functions ////

function bang()
{
    active = 1;
}

function jit_matrix()
{
    if (active != 1)
        return;
    active = 0;
    
    var i,j;
    var matrixname = "none"; 
    var file = new File(filename, "write");
    
    if (arguments.length>0)
        matrixname = arguments[0];
    
    if (arguments.length>1)
    {
        primitive = arguments[1];
        post("Received drawing primitive: " + primitive + "\n");
    } 
    else 
    {
        post("Assuming default primitive: " + primitive + "\n");
    }
    
    var mymatrix = new JitterObject("jit.matrix",matrixname);
    
    file.open();
    
    pc = mymatrix.planecount;
    post("planecount: " + mymatrix.planecount + '\n');

    matsize = mymatrix.dim[0]*mymatrix.dim[1];
    
        
    if (file.isopen) {
        post('writing to ' + filename + '\n');
        vertoffset = 0;
        
        if(hasmaterial == 1)
        {
            writeMaterialFile();
            file.writestring("mtllib " + mtlname + mtlext + "\n");
        }
        
        if(groupmode == 1)
        {
            vertoffset = getoffset(file);
        }
        
        if(face(1) == 0)
            return;
        
        //// write vertices ////
        
        var vertCounter = 0;
        
        if (pc >= 3)
        {
            for (i=0;i<mymatrix.dim[0];i++) {
                for (j=0;j<mymatrix.dim[1];j++) {
                    var cell = mymatrix.getcell(i,j);
                    var s = "v " + cell[0].toFixed(8) + " " + cell[1].toFixed(8) + " " + cell[2].toFixed(8) + "\n";
                    file.writestring(s);
                    
                }
            }
            file.writestring("\n");
        }
        
                //// write texture vertices ////

        if (pc >= 5)
        {
            for (i=0;i<mymatrix.dim[0];i++) {
                for (j=0;j<mymatrix.dim[1];j++) {
                    var cell = mymatrix.getcell(i,j);
                    var s = "vt " + cell[3].toFixed(8) + " " + cell[4].toFixed(8) + "\n";
                    file.writestring(s);
                }
            }
        }
        
        //// write vertice normals //// 
        
        if (pc >= 8)
        { 
            for (i=0;i<mymatrix.dim[0];i++) {
                for (j=0;j<mymatrix.dim[1];j++) {
                    var cell = mymatrix.getcell(i,j);
                    var s = "vn " + -cell[5].toFixed(8) + " " + -cell[6].toFixed(8) + " " + -cell[7].toFixed(8) + "\n";
                    file.writestring(s);
                }
            }
            file.writestring("\n");
        }
        
        //// object and group ////
        
        file.writestring("o " + matrixname + "_" + groupnum + "\n" 
                         + "g " + matrixname + "_" + groupnum + "\n"
                         + "s off\n\n");
        
        //// include material file ////
        
        if (hasmaterial == 1)
        {
            file.writestring("usemtl " + mtlname + "\n\n");
        }       
        
        //// write faces ////
        
        if(primitive == "tri_fan")
        {
            for(i=1; i<=mymatrix.dim[0];++i)
            {
                var s = "";
                for(j=2; j<mymatrix.dim[1];++j)
                {
                    pos = (i-1) * mymatrix.dim[0];
                    s += "f " + face(pos+1) + " " + face(j+pos) + " " + face (j+1+pos) + "\n";
                }
                file.writestring(s);
            }
        }
        
        else if(primitive == "tri_strip")
        {
            for(i=1; i<=matsize-2;i+=2)
            {
                while((i+1)%mymatrix.dim[1] != 0)
                {
                    var s = "f " + face(i) + " " + face(i+1) + " " + face(i+2);
                    file.writestring(s + "\n");
                    ++i;
                }
            }
        }
        
        else if(primitive == "triangle")
        {
            
            if (matsize %3 != 0)
            {
                post("Error: If you are using " + primitive + " as drawing primitive your matrix size should be a multiple of 3");
                return;
            }
            
            for(i=1;i<=matsize-2;)
            {
                var border = i + 2;
                var s = "f ";
                while(i<=border)
                {
                    s += face(i) + " ";
                    ++i;
                }
                s += "\n";
                file.writestring(s);
            }
        }
        
        else if(primitive == "quad_grid")
        {
            if(mymatrix.dim[1]%2 != 0)
            {
                post("Error: If you are using " + primitive + " as drawing primitive your matrix size should be a multiple of 4");
                return;
            }
            
            for(var i=1;i<mymatrix.dim[0]*(mymatrix.dim[1]-1);++i)
            {
                if(i%mymatrix.dim[1] == 0)
                    ++i;
                var s = "f " + face(i) + " " + face(i+mymatrix.dim[0]) + " " + face(i + mymatrix.dim[0]+1) + " " + face(i +1);
                //                      var s = "f " + face(i) + " " + face(i+1) + " " + face(i+2) + " " + face(i+3);
                file.writestring(s + "\n");
            }
        }
        
        else if(primitive == "tri_grid")
        {
            if(mymatrix.dim[1]%3 != 0)
            {
                post("Error: If you are using " + primitive + " as drawing primitive your matrix size should be a multiple of 3");
                return;
            }

            for(var i=1;i<mymatrix.dim[0]*(mymatrix.dim[1]-1);++i)
            {
                var s = "f " + face(i) + " " + face(i+mymatrix.dim[0]) + " " + face(i+1) + "\n";
                s += "f " + face(i+1) + " " + face(i+mymatrix.dim[0]) 
                    + " " + face(i+mymatrix.dim[0]+1) + "\n";
                file.writestring(s);
            }
        }
        
        else
        {
            post("The primitive \"" + primitive + "\" is not supported");
            return;
        }
        
        file.writestring("\n\n");
        file.writestring("#vertex count: " + (matsize+vertoffset) + "\n");
        
        file.eof = file.position;
        file.close();
    }
    else 
    {
        post("Error: could not open file\n");           
    }    
}

